import 'package:osam/domain/state/base_state.dart';
import 'package:osam/domain/usecase/usecase.dart';

class UseCaseNotFoundException implements Exception {
  final Type useCaseType;
  UseCaseNotFoundException(this.useCaseType);
}

class Store<ST extends BaseState> {
  final ST state;
  final List<UseCase<ST>> useCases;

  U useCase<U extends UseCase<ST>>() {
    final useCase = _useCasesMap[U];
    if (useCase == null) {
      throw UseCaseNotFoundException(U);
    } else
      return useCase;
  }

  final _useCasesMap = <Type, UseCase<ST>>{};

  Store(this.state, {this.useCases}) {
    if (useCases?.isNotEmpty ?? false) {
      for (final useCase in useCases) {
        useCase.store = this;
        _useCasesMap[useCase.runtimeType] = useCase;
      }
    }
  }

  void _setStoreToUseCases(List<UseCase<ST>> newUseCases) =>
      newUseCases.forEach((useCase) => useCase.store = this);

  void updateUseCases(List<UseCase<ST>> newUseCases) {
    _setStoreToUseCases(newUseCases);
    newUseCases.forEach((useCase) => _useCasesMap[useCase.runtimeType] = useCase);
  }
}
