part of 'navigation_state.dart';

abstract class Navigation {}

abstract class MainNavigation implements Navigation {
  StateStream<List<String>> get routes;

  void pop();

  void push(String route);

  void pushReplacement(String route);

  void pushAndRemoveUntil(String route);
}

abstract class TabNavigation {
  StateStream<String> get tab;

  void changeTab(String tab);
}

mixin NavigationPresenter<ST extends BaseState<ST>, US extends BaseState<US>> on Presenter<ST, US>
    implements MainNavigation {
  NavigationState get navigationState;

  StreamSubscription<List<String>> _routesSub;

  @override
  void init() {
    _routesSub = navigationState.routes.listen((routes) {});
  }

  @override
  StateStream<List<String>> get routes => navigationState.routes;

  @override
  void pop() => navigationState._pop();

  @override
  void push(String route) => navigationState._push(route);

  @override
  void pushAndRemoveUntil(String route) => navigationState._pushAndRemoveUntil(route);

  @override
  void pushReplacement(String route) => navigationState._pushReplacement(route);

  @override
  void dispose() {
    _routesSub.cancel();
    super.dispose();
  }
}

mixin TabNavigationPresenter<ST extends BaseState<ST>, US extends BaseState<US>>
    on Presenter<ST, US> implements TabNavigation {
  TabNavigationState get tabNavigationState;

  @override
  void changeTab(String tab) => tabNavigationState._changeTab(tab);

  @override
  StateStream<String> get tab => tabNavigationState.tab;
}
